<?php
// Exit if accessed directly
if (!defined('ABSPATH')) exit;


if( class_exists('EM_Ongoing_Events') ) {
	new EM_Ongoing_Events_Metabox();
}


class EM_Ongoing_Events_Metabox extends EM_Ongoing_Events {

	public function __construct() {
		add_action('add_meta_boxes', array($this, 'create_meta_box'));
		add_action('post_updated', array($this, 'save_meta_box_data'));
		add_filter('em_event_validate_meta',array($this, 'validate_ongoing_input'), 10, 2);
	}


	#===============================================
	# 	Initialize the Meta Box in the Event Admin Post.
	#===============================================
	public static function create_meta_box($post_type) {
		if( $post_type != 'event') {
			return;
		}
		// Only show this meta box to users with the right capabilities.
		if( $post_type === 'event' && is_admin() && current_user_can('publish_events', 'edit_events', 'edit_other_events') ) {
			$plugin = parent::get_plugin_data();
			add_meta_box('ongoing_event', esc_html__('Ongoing Event', $plugin['text']), array($this, 'show_meta_box'), 'event', 'normal', 'high');
		}
	}


	#===============================================
	# 	Render the Ongoing Meta Box.
	#===============================================
	public static function show_meta_box($post) {
		global $post, $EM_Event;
		$plugin 			= parent::get_plugin_data();
		$text 				= $plugin['text'];
		$get 				= parent::translate();
	    $count 				= 2; // Always start at 2 -> Main Event = 1.
		$ongoing_checkbox 	= get_post_meta($post->ID, 'ongoing_check', true);

		parent::load_assets();

		if( $EM_Event->is_recurring() || $EM_Event->is_recurrence() ) {
			echo '<span class="stonehenge-error">'. __('This is a Recurring Event.', $text) .'</span>';
			return;
		}

		if( event_is_ongoing() ) {
			// Check for shortcode.
			if(strpos($post->post_content, '[ongoing_events]') === false) {
				$notice = sprintf( esc_html__('Please make sure to use the shortcode %s in your main content.', $text), '<code>[ongoing_events]</code>');
				echo parent::show_notice( '<b>'.$notice.'</b>', 'error', false);
			}

			if( event_has_parent() ) {
				$parent_id 			= event_get_parent_id();
				$ongoing_this 		= get_post_meta($post->ID, 'ongoing_day', true);
				$notice 			= '<strong>' . sprintf( _x('This is %1$s %2$s of <a href=%3$s>%4$s</a>.', 'This is item # of Main Event', $text),
					esc_html($get['item']), esc_html($ongoing_this), esc_url_raw(get_edit_post_link($parent_id)), esc_html(get_the_title($parent_id)) ) .'</strong><br>' . esc_html__('Sub Event Details and Bookings are maintained in the Main Event.', $text);
				echo $notice;

				// Show Admin Notice.
				echo parent::show_notice($notice, 'warning', false);

				// Hide Delete, Bookings, Comments.
				?><script>
					jQuery('.em-date-start, .em-date-end, #start-time, #end-time, #em-time-all-day, #event-timezone').attr('disabled', true);
					jQuery('#event-date-explanation').html('<?php echo $get['maintain']; ?>').css('color', '#e14d43');
				</script>
				<style>.submitdelete, .deletion, .misc-event-duplicate-link, #em-event-bookings, #commentstatusdiv, #commentsdiv, #slugdiv, #edit-slug-buttons {display:none !important;}</style><?php
				return;
			}

			else {
				// Show Admin Notice for Main Event.
				$notice	= sprintf( '<strong>%s</strong><br>%s %s',
			 		__('This is the main event of an ongoing event series.', $plugin['text']),
			 		__('Dates and booking are maintained here.', $plugin['text']),
			 		__('If you delete this event, all linked sub-events will be deleted as well.', $plugin['text'])
			 	);
				echo parent::show_notice($notice, 'warning', false);
			}
		}

		// Proceed to output the meta box for Main Event & Single Events.
		wp_nonce_field( plugin_basename( __FILE__ ), '_em_ongoing_nonce' );
		?>
		<p id="ongoing_ask"><strong><?php _e('Is this an Ongoing Event?', $text); ?></strong>&nbsp;&nbsp;
		<label class="toggle">
			<input type="checkbox" id="ongoing_check" name="ongoing_check" value="yes" <?php if($ongoing_checkbox) echo 'checked="checked"'; ?>>
			<span data-unchecked="<?php echo esc_attr( __('No'), ENT_QUOTES); ?>" data-checked="<?php echo esc_attr( __('Yes'), ENT_QUOTES); ?>"></span>
		</label>
		</p>
		<div id="ongoing-events" style="display:none;">
			<table id="repeatable-fieldset-one" class="stonehenge-metabox">
				<thead>
					<th></th>
					<th><?php echo esc_html($get['date']); ?></th>
					<th><?php echo esc_html($get['time']); ?></th>
					<th></th>
					<th></th>
				</thead>
				<tbody id="here">
				<?php
				$ongoing_events = event_get_linked_events($post);
				if( count( (array) $ongoing_events) > 0 ) {
					foreach($ongoing_events as $event ) {
						$event_start_date 	= get_post_meta($event->ID, '_event_start_date', true);
						$event_start_time 	= get_post_meta($event->ID, '_event_start_time', true);
						$event_end_time 	= get_post_meta($event->ID, '_event_end_time', true);

						echo sprintf( '<tr id="day%1$s"><td align="left"><strong>%2$s %1$s</strong><input type="hidden" name="ongoing_events[%1$s][day]" value="%1$s" autocomplete="off" required="required"></td><td align="center"><input type="text" name="ongoing_events[%1$s][date]" value="%3$s" class="session-date" autocomplete="off" size="22" required="required"></td><td align="center"><input type="text" name="ongoing_events[%1$s][start]" value="%4$s" class="em-time-input em-time-start" size="8" autocomplete="off" required="required"> – <input type="text" name="ongoing_events[%1$s][end]" value="%5$s" class="em-time-input em-time-end" size="8" autocomplete="off" required="required"></td><td align="right"><span class="remove" title="%8$s">&odash;</span>&nbsp;<span class="add" title="%9$s">&oplus;</span><input type="hidden" name="ongoing_events[%1$s][ID]" value="%6$s"></td><td><span class="edit">%7$s</span></td></tr>',
							esc_attr($count, ENT_QUOTES), esc_attr($get['item'], ENT_QUOTES),
							esc_attr(date_i18n($get['date_format'], strtotime($event_start_date)), ENT_QUOTES),
							esc_attr(date_i18n($get['time_format'], strtotime($event_start_time)), ENT_QUOTES),
							esc_attr(date_i18n($get['time_format'], strtotime($event_end_time)), ENT_QUOTES),
							esc_attr($event->ID, ENT_QUOTES),
							wp_kses_allowed('<a href="'. get_edit_post_link( $event->ID) .'" target="_blank" title="'. esc_html__('Edit') .'">' . '&#9998;' .'</a>'), __('Remove'), __('Add') );
						$count = $count + 1;
					} // end foreach
				} // end if(count > 0)
				else {
					// Create the first, empty input group.
					echo sprintf( '<tr id="day%1$s"><td align="left"><strong>%2$s %1$s</strong><input type="hidden" name="ongoing_events[%1$s][day]" value="%1$s" autocomplete="off"></td><td align="center"><input type="text" name="ongoing_events[%1$s][date]" value="" class="session-date first" autocomplete="off" size="22"></td><td align="center"><input type="text" name="ongoing_events[%1$s][start]" value="" class="em-time-input em-time-start first" size="8" autocomplete="off"> – <input type="text" name="ongoing_events[%1$s][end]" value="" class="em-time-input em-time-end first" size="8" autocomplete="off"></td><td align="right"><span class="remove" title="%3$s">&odash;</span>&nbsp;<span class="add" title="%4$s">&oplus;</span></td><td></td></tr>', $count, $get['item'], __('Remove'), __('Add') );
					$count = $count + 1;
				}
				?>
				</tbody>
			</table>
		</div>
		<!-- Send current $count to jQuery. -->
		<script>var Count = '<?php echo $count; ?>';
		var Days = (Count -1) ;</script>
		<?php
	}


	#===============================================
	# 	Sanitize form input & save data to SQL database.
	#===============================================
	public static function save_meta_box_data($post_id) {

		if( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE ) {	return; }
		if( !isset($_POST['_em_ongoing_nonce']) ) { return; }
		if( !wp_verify_nonce($_POST['_em_ongoing_nonce'], plugin_basename( __FILE__ )) ) { return; }

 		// Check if Ongoing Checkbox is checked.
 		if( !isset($_POST['ongoing_check']) || !isset($_POST['ongoing_events'])  ) {

 			// First check of there are sub-events created in the database already.
 			$existing_events = event_get_linked_events();
 			if( !empty($existing_event) ) {
	 			foreach( $existing_events as $existing_event ) {
					wp_delete_post( $existing_event->ID, true); //Bypass Trash.
	 			}
 			}
 			// Delete meta keys, just in case.
			delete_post_meta($_POST['post_ID'], 'ongoing_events');
			delete_post_meta($_POST['post_ID'], 'ongoing_check');
			return;
 		}
	 	else {
			$count = 2; // Always start at 2 => Main Event = 1.
			$ongoing_events = array();

			// First, sort & clean the incoming data.
			$cleaned_events = $_POST['ongoing_events'];
			usort($cleaned_events, 'sort_events_by_date');

			foreach( $cleaned_events as $events => $event ) {
				if( strtotime($event['start']) < strtotime($event['end']) ) {
					$ongoing_events[$count]['day'] 		= sanitize_text_field( (int) $count); // Always fill with new to keep it clean.
					$ongoing_events[$count]['date'] 	= sanitize_text_field( date('Y-m-d', strtotime($event['date'])) );
					$ongoing_events[$count]['start'] 	= sanitize_text_field( date('H:i:s', strtotime($event['start'])) );
					$ongoing_events[$count]['end'] 		= sanitize_text_field( date('H:i:s', strtotime($event['end'])) );

					if( isset($event['ID']) && !empty($event['ID']) ) {
						$ongoing_events[$count]['ID'] 		= sanitize_text_field( (int) $event['ID']);
					}
					$count = $count + 1;
				}
			}
			update_post_meta($_POST['post_ID'], 'ongoing_events', $ongoing_events);
			update_post_meta($_POST['post_ID'], 'ongoing_check', sanitize_key($_POST['ongoing_check']));
			return;
		}
		return;
	}


	#===============================================
	# 	Validate ongoing input before saving to database.
	#===============================================
	public static function validate_ongoing_input( $result, $EM_Event ) {
		$plugin = parent::get_plugin_data();
		$saved 	= get_option( $plugin['slug'] );
		$get 	= parent::translate();

		if( $_POST['post_type'] === 'event' && isset($_POST['ongoing_check']) && isset($_POST['ongoing_events']) ) {
			foreach( $_POST['ongoing_events'] as $events => $event ) {
				foreach($event as $key) {
					if( strtotime($event['start']) > strtotime($event['end']) ) {	// Use strtotime because of AM/PM.
						$message = null;
						$message = sprintf('<strong>%1$s %2$s:</strong> %3$s',
							$get['item'], $event['day'], __('The end time cannot before the start time.', $plugin['text'])
						);
						$EM_Event->add_error( $message );
						return false;
					}
					if( empty($key) ) {
						// Fallback, because fields are all required in the form.
						$EM_Event->add_error( __('You need to enter at least one full date to create an ongoing event.', $plugin['text']) );
						return false;
					}
				}
			}
		}
	  return $result;
	}

} // End class.
