<?php
// Exit if accessed directly
if (!defined('ABSPATH')) exit;

include_once(ABSPATH.'wp-admin/includes/plugin.php');

if( class_exists('EM_Ongoing_Events') ) {
	new EM_Ongoing_Events_Process();
}


#===============================================
# 	Define the Main Class of this plugin.
#===============================================
class EM_Ongoing_Events_Process extends EM_Ongoing_Events {

	public function __construct() {
		add_action('before_delete_post', array($this, 'maybe_delete_subevents'));
		add_action('save_post', array($this, 'process_ongoing_values'), 10, 2);
		add_filter('post_row_actions', array($this, 'hide_actions'), 30, 2 );
		add_filter('page_row_actions', array($this, 'hide_actions'), 30, 2 );
		add_filter('manage_event_posts_custom_column', array($this, 'add_descripton_to_column_output'), 10, 2);

		add_filter('manage_edit-event_sortable_columns', array($this, 'change_ongoing_title'));
		add_shortcode('ongoing_events', array($this, 'show_ongoing_events_table'));
		add_shortcode('ongoing-events', array($this, 'show_ongoing_events_table'));
	}


	#===============================================
	# 	Determine what needs to be done when the Main Event is updated.
	#===============================================
	public static function process_ongoing_values($parent_id, $post) {

		// Only continue if this the Main Event of an Ongoing Series. (Sub events do not have the nonce.)
		if( $post->post_type != 'event' || $post->post_status != 'publish' || !isset( $_REQUEST['_em_ongoing_nonce'] ) ) {
			return;
		}

		// Manually check for ongoing => Is this a parent?
		$checkbox 	= get_post_meta($post->ID, 'ongoing_check', true);
		if( $post->post_type === 'event' && $post->post_parent === 0 && $checkbox === 'yes' ) {
			$ongoing_events 	= get_post_meta($parent_id, 'ongoing_events', true);
			$existing_events 	= event_get_linked_events();

			// Create an array with ID's of Sub Events in the Main Event.
			if( !empty($ongoing_events) ) {
				$ongoing_ids = array();
				foreach($ongoing_events as $ongoing_event) {
					$ongoing_ids[] = @$ongoing_event['ID'];
				}
			}

			// Create an array with ID's of existing Sub Events.
			if( !empty($existing_events) ) {
				$existing_ids = array();
				foreach($existing_events as $existing_event) {
					$existing_ids[] = $existing_event->ID;
				}
			}

			//------------------------------------------------------
			// 	Start processing.
			//------------------------------------------------------
			if( empty($ongoing_events) && empty($existing_events) ) {
				// This is and was not an Ongoing Main Event.
			}

			if( empty($ongoing_events) && !empty($existing_events) ) {
				// Delete all left-overs.
				foreach($existing_ids as $id) {
					wp_delete_post($id, true);
				}
			}

			if( !empty($ongoing_events) && empty($existing_events) ) {
				// We are creating a new series, so create them all.
				foreach($ongoing_events as $event) {
					self::create_ongoing_event($event);
				}
			}

			if( !empty($ongoing_events) && !empty($existing_events) ) {
				// There is stuff there already -> Add of modify?
				foreach($ongoing_events as $event) {
					if( isset($event['ID']) && !empty($event['ID']) && in_array($event['ID'], $existing_ids) ) {
						self::update_ongoing_event($event);
					}
					elseif( !isset($event['ID']) || empty($event['ID']) ) {
						self::create_ongoing_event($event);
					}
				}
				// Delete sub event from sequence if manually removed from the series.
				foreach($existing_ids as $id) {
					if( !in_array($id, $ongoing_ids) ) {
						wp_delete_post($id, true);
					}
				}
			}
		}

		// Double check.
		if( !event_is_ongoing() && !event_has_parent() ) {
			$existing_events 	= event_get_linked_events();
			if( !empty($existing_events) ) {
				// Remove left-overs.
				foreach($existing_events as $existing_event) {
					wp_delete_post($existing_event->ID, true);
				}
			}
		}
		return;
	}


	#===============================================
	# 	Create a new Sub Event.
	#===============================================
	public static function create_ongoing_event($event) {

		// Are we really creating a new event? => Double check, again.
		if( isset($event['ID']) || !empty($event['ID']) ) {
			return;
		}

		$plugin 		= parent::get_plugin_data();
		$text 			= $plugin['text'];
		$saved 			= get_option( $plugin['slug'] );
		$get 			= parent::translate();
		$parent_id 		= event_get_parent_id();
		$parent_title 	= get_the_title($parent_id);
		$new_title 		= $parent_title .' ('. $get['item'] .' '. $event['day'] .')';
		$new_slug 		= sanitize_title_for_query( sprintf('%s-%s', $get['item'], $event['day']) );

		$current_UTC 	= date('Y-m-d H:i:s', time() );
		$current_local 	= Stonehenge_Plugin::convert_to_local($current_UTC);

		if( !get_event_by_slug($new_slug) ) {
			$saved 			= get_option( $plugin['slug'] );
			$content 		= apply_filters('em_ongoing_wildcards', $saved['strings']['sub_content'], $event, $parent_id);
			$excerpt  		= apply_filters('em_ongoing_wildcards', $saved['strings']['sub_excerpt'], $event, $parent_id);

			// First, create a new EM_Event Object.
			$Main 			= new EM_Event($parent_id, 'post_id');
			$New			= $Main;

			// Reset certain data.
			if( get_option('dbem_categories_enabled') ) $New->get_categories(); // Before we remove event/post ID's.
			$New->ID 				= null;
			$New->post_id 			= null;
			$New->event_id 			= null;
			$New->post_name			= $new_title;
			$New->name 				= $new_title;
			$New->event_name 		= $new_title;
			$New->event_slug 		= $new_slug;
			$New->slug 				= $new_slug;
			$New->post_content 		= $content;
			$New->post_excerpt 		= $excerpt;
			$New->event_start_date	= $event['date'];
			$New->start_date		= $event['date'];
			$New->event_end_date	= $event['date'];
			$New->end_date			= $event['date'];
			$New->event_start_time	= $event['start'];
			$New->start_time		= $event['start'];
			$New->event_end_time	= $event['end'];
			$New->end_time			= $event['end'];
			$New->event_all_day		= 0;
			$New->event_rsvp 		= 0;
			$New->rsvp 				= 0;
			$New->force_status 		= 'publish';
			$New->save();

			$new_post_id =  $New->post_id;
			// Make the new post hierarchical.
			$new_post = array(
				'ID' 				=> $new_post_id,
				'post_parent' 		=> $parent_id,
				'post_status' 		=> 'publish',
				'comment_status'	=> 'closed',
				'ping_status'		=> 'closed',
				'meta_input' 		=> array(
					'ongoing_day' 		=> $event['day'],
				),
				'menu_order'		=> $event['day'],
				'post_modified' 	=> $current_local,
				'post_modified_gmt'	=> $current_UTC,
			);
			wp_update_post( $new_post );

			// Add extra meta to Post.
			$meta['_event_start_date'][0] 	= $event['date'];
			$meta['_event_end_date'][0] 	= $event['date'];
			$meta['_event_start_time'][0] 	= $event['start'];
			$meta['_event_end_time'][0] 	= $event['end'];
			$meta['_event_start'][0] 		= Stonehenge_Plugin::convert_to_UTC( $event['date'] .' '. $event['start'], 'Y-m-d H:i:s');
			$meta['_event_end'][0] 			= Stonehenge_Plugin::convert_to_UTC( $event['date'] .' '. $event['end'], 'Y-m-d H:i:s');
			$meta['_event_all_day'][0] 		= 0;
			$meta['_event_timezone'][0] 	= get_post_meta($parent_id, '_event_timezone', true);
			$meta['_event_rsvp'][0] 		= 0;
			$meta['_event_rsvp'][0] 		= 0;
			$meta['_event_start_local'][0] 	= $event['date'] .' '. $event['start'];
			$meta['_event_end_local'][0] 	= $event['date'] .' '. $event['end'];
			$meta['_thumbnail_id'][0] 		= get_post_meta($parent_id, '_thumbnail_id', true);

			// Insert post meta for new event.
			foreach($meta as $meta_key => $meta_vals) {
				foreach($meta_vals as $meta_val) {
				    update_post_meta($new_post_id, $meta_key, $meta_val);
				}
			}

			// Clean the new Event Object.
			$remove_meta = array( '_event_rsvp_date', '_event_rsvp_time', '_event_rsvp_spaces', '_event_spaces', 'ongoing_events', 'ongoing_check', '_em_active_gateways', '_event_all_day', '_recurrence', '_recurrence_days' );

			foreach( $remove_meta as $key ) {
				delete_post_meta($new_post_id, $key);
			}

			// Update the SQL for this event -> Dates and times tend to keep on the old values.
			global $wpdb;
			$query = self::construct_query_from_meta( $meta, $new_post_id );
			$sql = $wpdb->query($query);

			// Restore Tickets to Main Event.
			self::restore_tickets_to_main_event( $new_post_id );

		}
		return;
	}


	#===============================================
	# 	Update existing Sub Event with new meta data.
	#===============================================
	public static function update_ongoing_event($event) {
		// $event = single array of one sub event with full data. Check with database to update both $post & $EM_Event.

		if( !isset($event['ID']) ||empty($event['ID']) ) {		// A Double check never hurts.
			return;
		}

		$id 		= $event['ID']; // Just easier.
		$plugin 	= parent::get_plugin_data();
		$get 		= get_option( $plugin['slug'] );
		$parent_id 	= event_get_parent_id();
		$timezone 	= get_post_meta($parent_id, '_event_timezone', true);

		$meta['_event_start_date'][0] 	= $event['date'];
		$meta['_event_end_date'][0] 	= $event['date'];
		$meta['_event_start_time'][0] 	= $event['start'];
		$meta['_event_end_time'][0] 	= $event['end'];
		$meta['_event_all_day'][0] 		= 0;
		$meta['_event_start'][0] 		= Stonehenge_Plugin::convert_to_UTC( $event['date'] .' '. $event['start'], 'Y-m-d H:i:s' );
		$meta['_event_end'][0] 			= Stonehenge_Plugin::convert_to_UTC( $event['date'] .' '. $event['end'], 'Y-m-d H:i:s' );
		$meta['_event_timezone'][0] 	= $timezone;
		$meta['_event_rsvp'][0] 		= 0;
		$meta['_event_rsvp'][0] 		= 0;
		$meta['_event_start_local'][0] 	= $event['date'] .' '. $event['start'];
		$meta['_event_end_local'][0] 	= $event['date'] .' '. $event['end'];
		// Do not update the Post Thumbnail, as it might have been manually changed in the meantime.

		// Compare new data with database.
		$post 		= get_post($id);
		$old_day 	= get_post_meta($id, 'ongoing_day', true);
		$old_date 	= get_post_meta($id, '_event_start_date', true);
		$old_start 	= get_post_meta($id, '_event_start_time', true);
		$old_end 	= get_post_meta($id, '_event_end_time', true);

		// Check if there is anything changes.
		if( $old_day != $event['day'] || $old_date != $event['date'] || $old_start != $event['start'] || $old_end != $event['end'] ) {

			// Update post meta.
			foreach($meta as $meta_key => $meta_vals) {
				foreach($meta_vals as $meta_val) {
				    update_post_meta($id, $meta_key, $meta_val);
				}
			}
			// Update database wp_em_events accordingly.
			global $wpdb;
			$query 	= self::construct_query_from_meta( $meta, $id );
			$sql 	= $wpdb->query($query);
		}

		// Do we need to change Post Details?
		if( $post->menu_order != $event['day'] ) {
			// Update a post with new post data, to keep correct order in Ongoing Events Table.
			$new_data = array(
				'menu_order'	=> $event['day'],
				'meta_input' 		=> array(
					'ongoing_day' 		=> $event['day'],
				),
			);
			wp_update_post( $new_data );
		}

		// Just in case...
		self::restore_tickets_to_main_event( $id );
		return;
	}


	#===============================================
	# 	Make updating the Events Table easier.
	#===============================================
	public static function construct_query_from_meta( $meta, $id ) {

		// Prepare meta inputs for correct array.
		$sql = array();
		foreach($meta as $meta_key => $meta_vals) {
			foreach($meta_vals as $meta_val) {
				$column = ltrim($meta_key, '_');
				$sql[$column] = $meta_val;
			}
		}

		// Unset values, because these are not used in the SQL Table.
		unset($sql['event_start_local']);
		unset($sql['event_end_local']);
		unset($sql['thumbnail_id']);

		// Construct SQL Query.
		$table 		= EM_EVENTS_TABLE;			// This constant takes prefix into account by default.
		foreach($sql as $column => $value ) {
			$value = "'$value'";
			$updates[] = "$column = $value";
		}
		$implode 	= implode(', ', $updates);
		$query 		= "UPDATE {$table} SET ". $implode ." WHERE post_id = '{$id}'";

		return $query;
	}


	#===============================================
	# 	Restore Tickets back to Main Event.
	#===============================================
	public static function restore_tickets_to_main_event( $new_post_id ) {
		global $wpdb;
		$parent_id 			= event_get_parent_id();
		$parent_event_id 	= get_post_meta($parent_id, '_event_id', true);
		$events_table 		= EM_EVENTS_TABLE;
		$tickets_table 		= EM_TICKETS_TABLE;
		$new_event_id 		= $wpdb->get_var("SELECT `event_id` FROM {$events_table} WHERE `post_id` = '{$new_post_id}'");
		$wrong_ids			= $wpdb->get_results("SELECT `ticket_id` FROM {$tickets_table} WHERE `event_id` = '{$new_event_id}'", ARRAY_A);
		if( count( (array) $wrong_ids) > 0 ) {
			foreach( $wrong_ids as $wrong ) {
				$right_id = $wpdb->query("UPDATE {$tickets_table} SET `event_id` = '{$parent_event_id}' WHERE `ticket_id` = '{$wrong['ticket_id']}'");
			}
		}
		return;
	}


	#===============================================
	# 	Shortcode: Output HTML table to list all ongoing events.
	#===============================================
	public static function show_ongoing_events_table() {
		$plugin = parent::get_plugin_data();
		$get 	= parent::translate();

		if( !event_is_ongoing() ) {
			return __('This is not an ongoing event.', $plugin['text']);
		}

		$parent_id 		= event_get_parent_id();
		$linked_events 	= event_get_linked_events();

		// First construct line 1 for Day 1 (Main Event).
		$EM_Main 		= em_get_event($parent_id, 'post_id');
		$main_url 		= get_permalink($EM_Main->post_id);
		$row_one 		= $EM_Main->output("<tr><td><strong><a href='{$main_url}' title='#_EVENTNAME'>{$get['item']} 1</a></strong></td><td>#_EVENTDATES</td><td>#_EVENTTIMES</td></tr>");

		// Start output.
		$table 	= null;
		$table 	.= '<h3>' . $get['shortcode_title'] . '</h3>';
		$table	.= '<table class="ongoing-events-table">
			<thead>
				<th></th>
				<th>'. $get['date'] .'</th>
				<th>'. $get['time'] .'</th>
			</thead>
			<tbody>';
		$table .= $row_one;

		// Loop through sub-events.
		foreach( $linked_events as $linked_event ) {
			$event 	= new EM_Event($linked_event->_event_id);
			$nr 	= $event->event_attributes['ongoing_day'];
			$url 	= get_permalink($event->post_id);
			$row 	= "<tr><td><strong><a href='{$url}' title='#_EVENTNAME'>{$get['item']} {$nr}</a></strong></td><td>#_EVENTDATES</td><td>#_EVENTTIMES</td></tr>";

			$table 	.= $event->output($row);
		}
		$table 	.= '</tbody></table>';
		$table 	.= '<p class="ongoing-events-notice">' . $get['booking_notice'] . '</p>';
		return $table;
	}


	#===============================================
	# 	Prevent deletion of sub events from WP_List Events Table.
	#===============================================
	public static function hide_actions($actions, $post) {
		if( isset($_REQUEST['post_type']) && 'event' == $_REQUEST['post_type']) {
			if( !empty($post->post_parent) ) {
				unset( $actions['trash'] );
				unset( $actions['delete'] );
				unset( $actions['duplicate'] );
			}
		}
		return $actions;
	}


	#===============================================
	# 	Change Sub Event Title in Events Posts List.
	#===============================================
	public static function change_ongoing_title( $sortable ) {
	    add_filter('the_title', array($this, 'add_ongoing_icon_to_tile'), PHP_INT_MAX);
	    return $sortable;
	}


	function add_ongoing_icon_to_tile($title) {
		global $post;
		if( $post->post_type === 'event' && $post->post_parent > 0 ) {
			$title = '&#x1F517; ' . $title;
		}
	    return $title;
	}


	#===============================================
	# 	Add text to 'extra' column in WP_List Events Table.
	#===============================================
	public static function add_descripton_to_column_output( $column ) {
		global $post, $EM_Event;
		switch ( $column ) {
			case 'extra':
				if( event_is_ongoing() ) {
					$plugin = parent::get_plugin_data();
					$text 	= $plugin['text'];
					$saved 	= get_option( $plugin['slug'] );
					$get 	= parent::translate();
					if( event_has_parent() ) {
						$parent_id = event_get_parent_id();
						$parent_post = get_post($parent_id);
						echo sprintf( '&#x1F517; %s %s &rArr; <strong><a href="%s">%s</a></strong>.<br>',
							ucfirst($get['item']),
							get_post_meta($post->ID, 'ongoing_day', true),
							get_edit_post_link( $parent_post->ID),
							$parent_post->post_title
						);
					}
					else {
						echo esc_html__('This is a Main Event of an ongoing event series.', $text) .' &#x1F517;<br>';
					}
				}
			break;
		}
	}


	public static function maybe_delete_subevents( $parent_id ) {
		global $post_type, $post;
		if( $post_type != 'event' ) {
			return;
		}

		// Get subevents.
		$sub_events = event_get_linked_events();
		if( !empty($sub_events) ) {
			foreach( $sub_events as $sub ) {
				wp_delete_post( $sub->post_id, true ); 	// Bypass Trash.
			}
		}
	}

} // End class.


#===============================================
# 	Check if this is an Ongoing Event.
#===============================================
if( !function_exists('event_is_ongoing') ) {
	function event_is_ongoing() {
		global $pagenow, $post;
		if( $pagenow === 'post.php' && $_REQUEST['action'] === 'edit' || $post->post_type === 'event' ) {
			$checkbox = get_post_meta($post->ID, 'ongoing_check', true);
			if( ($post->post_parent == 0 && $checkbox === 'yes') || ($post->post_parent > 0) ) {
				return true;
			}
		}
		return false;
	}
}

#===============================================
# 	Check if this event is a page child / Sub-Event.
#===============================================
if( !function_exists('event_has_parent') ) {
	function event_has_parent() {
		global $post, $pagenow;

		if( $pagenow != 'post.php' && $pagenow != 'edit.php') {
			return;
		}

		if( event_is_ongoing() &&  $post->post_parent > 0) {
			return true;
		}

		return false;
	}
}


#===============================================
# 	Fetch the Post ID of this Sub-Event.
#===============================================
if( !function_exists('event_get_parent_id') ) {
	function event_get_parent_id() {
		global $post;
		$parent_id = $post->post_parent ? $post->post_parent : $post->ID;
		return $parent_id;
	}
}


#===============================================
# 	Collect all Sub-Events of an Ongoing Series.
#===============================================
if( !function_exists('event_get_linked_events') ) {
	function event_get_linked_events() {
		global $post;
		$parent_id = $post->post_parent ? $post->post_parent : $post->ID;
		$args = array(
			'post_type' 	=> 'event',
			'post_parent' 	=> $parent_id,
			'numberposts'	=> -1,
			'post_status' 	=> 'any',
			'orderby' 		=> 'menu_order',
			'order' 		=> 'ASC',
		);
		$linked_events = get_children($args);
		return $linked_events;
	}
}

#===============================================
# 	Sort Ongoing Events by Date after being set in the Main Event.
#===============================================
if( !function_exists('sort_events_by_date') ) {
	function sort_events_by_date($a, $b) {
		return strcmp($a["date"], $b["date"]);
	}
}


#===============================================
# 	Find the unique Post by Event Slug.
#===============================================
if( !function_exists('get_event_by_slug') ) {
	function get_event_by_slug( $page_slug ) {
		global $wpdb;
		$table 	= EM_EVENTS_TABLE;
		$result = $wpdb->get_var("SELECT `post_id` FROM {$table} where `event_slug` = '{$page_slug}'");

		if( $result ) {
			return $result;
		}
		return false;
	}
}
