<?php
if( !defined('ABSPATH') ) exit;
if( !defined('STONEHENGE') ) define('STONEHENGE', 'https://www.stonehengecreations.nl/');

if( !class_exists('Stonehenge_License') ) {
	class Stonehenge_License {

		private static $plugin;
		private static $instance;


		#===============================================
		public static function init($plugin) {
			$plugin['text'] 		= isset($plugin['text']) ? $plugin['text'] : str_replace('_', '-', $plugin['slug']);
			$plugin['slug'] 		= isset($plugin['slug']) ? $plugin['slug'] : str_replace('_', '-', $plugin['text']);
			$plugin['icon'] 		= isset($plugin['icon']) ? trim($plugin['icon']).'&nbsp;' : '';
			$plugin['version_slug'] = $plugin['base'].'_version';
			$plugin['license_slug']	= $plugin['base'].'_license';
			$plugin['license'] 		= get_option( $plugin['license_slug'] );
			self::$plugin = $plugin;
			if( @self::$instance->plugin['base'] != $plugin['base'] ) {
				self::$instance = new Stonehenge_Plugin($plugin);
			}
			return self::$instance;
		}


		#===============================================
		public static function is_valid( $plugin ) {
			$license 		= $plugin['license'];
			$is_licensed 	= self::is_licensed($plugin);
			$is_valid 		= false;

			$is_valid = (!$is_licensed || (isset($license['license']) && $license['license'] === 'valid') ) ? true : false;

			if( $is_valid ) {
				do_action('stonehenge_loaded');
			}
			return $is_valid;
		}


		#===============================================
		public static function is_licensed( $plugin ) {
			$license 		= $plugin['license'];
			$license_slug 	= $plugin['license_slug'];

			if( !$license || empty($license) ) {
				$response 	= json_decode( file_get_contents( STONEHENGE . '?api=edd&base='. $plugin['base'] ) );
				$check 		= $response->licensed ? $response->id : 'no';

				if( !add_option( $license_slug, $check, '', 'no') ) {
					update_option( $license_slug, $check, 'no');
				}
				$is_licensed = $response->licensed;
			}
			elseif( $license ) {
				if( !is_array($license) ) {
					$is_licensed = ($license != 'no') ? true : false;
				} else {
					$is_licensed = true;
				}
			}

			if( $is_licensed ) {
				if( wp_next_scheduled('puc_cron_updater-'. $plugin['base']) ) {
					add_action('puc_cron_updater-'. $plugin['base'], array(__CLASS__, 'validate'), 8);
				}
			}
			return $is_licensed;
		}


		#===============================================
		public static function show_license( $plugin ) {
			$text 			= $plugin['text'];
			$saved 			= $plugin['license'];
			$show 			= self::is_valid($plugin) ? 'closed' : '';

			if( isset($_POST['activate_license']) ) {
				self::activate( $_POST['license_key'], $plugin );
				echo '<meta http-equiv="refresh" content="0">';
			}

			if( isset($_POST['check_license']) ) {
				self::check( $saved['license_key'], $plugin );
				echo '<meta http-equiv="refresh" content="0">';
			}

			if( isset($_POST['deactivate_license']) ) {
				self::deactivate( $saved['license_key'], $plugin );
				echo '<meta http-equiv="refresh" content="0">';
			}
			?>
			<div class="meta-box-sortables ui-sortable" id="normal-sortables">
				<div class="postbox stonehenge-metabox <?php echo $show; ?>" id="license">
					<h3 class="hndle"><?php esc_html_e('License Key', $text); ?></h3>
					<div class="inside">
						<form action="" method="post">
							<table class="stonehenge-table">
								<tr>
									<td colspan="2"><p>You need a valid license key to use this plugin and receive updates and support</p></td>
								</tr>
								<tr>
									<th>License Status:</th>
									<?php
									if( !is_array($saved) ) {
										echo '<td><span class="pro">Please enter your License Key</span></td></tr><tr><th>License Key</th>';
										echo '<td><input type="text" name="license_key" id="license_key_'. esc_attr($plugin['base']) .'" value="" class="regular-text"></td></tr>';
										echo '<tr><th></th>';
										echo '<td><input type="submit" name="activate_license" value="Activate License Key" class="button-primary">&nbsp;&nbsp;</td></tr>';
									}
									else {
										$status 		= $plugin['license']['license'];
										$type 			= ($saved['license_limit'] === 1) ? 'Single ' : $saved['license_limit'] .'-';
										$date_format 	= get_option('date_format');
										$class 			= ($status != 'valid') ? 'error' : 'success';

										echo '<td><span class="'.$class.'">'. str_replace('_', ' ' , ucfirst($status)) .'</span></td></tr>';

										switch( $status ) {
											case 'valid':
												echo '<tr><th>License Key:</strong></td><td><strong>'. self::mask($saved['license_key']) .'</strong></th></tr>';
												echo '<tr><th>Licensed to:</strong</th>';
												echo '<td>'. $saved['customer_name'] .'<br>'. $saved['customer_email'] .'</td></tr>';
												echo '<tr><th>License Type:</th><td>'. $type .'Site License ';
												echo "<em>({$saved['site_count']} used)</em></td></tr>";
												echo '<tr><th>Expires on:</th>';
												echo '<td>'. date_i18n( $date_format, strtotime( $saved['expires'], current_time('timestamp') ) ) .'</td></tr>';
												echo '<tr><td></td><td><input type="submit" name="check_license" value="&#8634; Sync Data" class="stonehenge-button">&nbsp;&nbsp;<input type="submit" name="deactivate_license" value="Deactivate License" class="button-secondary"></td></tr>';
											break;
											case 'expired':
												echo '<tr><th></th><td class="red">Your license key expired on '.date_i18n( $date_format, strtotime( $saved['expires'], current_time('timestamp') ) ) .'.</td></tr>';
												echo '<tr><th>10% Renewal Discount:</th><td><a href="'. self::get_renewal_url() .'" target="_blank"><button type="button" class="stonehenge-button">Renew License</button></a></td></tr>';
											break;
											case 'disabled':
												echo '<tr><th></th><td>'. sprintf('You need to buy a <a href=%s target="blank">new License Key</a> or delete this plugin.', STONEHENGE .'creations/') .'</td></tr>';
												echo '<tr><td></td><td><input type="submit" name="deactivate_license" value="Deactivate License" class="stonehenge-button"></td></tr>';
											break;
											case 'invalid':
											default:
											echo '<tr><th>License Key:</th>';
											echo '<td><input type="text" name="license_key" id="license_key" value="'. @$saved['license_key'] .'" class="regular-text"></td></tr>';
											echo '<tr><th></th><td><input type="submit" name="activate_license" value="Activate" class="stonehenge-button">&nbsp;&nbsp;</td></tr>';
											break;
										}
									}
									?>
								</tr>
							</table>
						</form>
					</div>
				</div>
			</div>
			<?php
		}


		#===============================================
		public static function license_server( $license_key, $method, $plugin ) {
			$license 	= $plugin['license'];
			$plugin_id 	= is_array($license) ? $license['item_id'] : $license;
			$parameters = array(
				'edd_action' 		=> $method,
				'item_id' 			=> $plugin_id,
				'license' 			=> trim($license_key),
				'url'				=> $_SERVER['SERVER_NAME'],
			);
			$response 		= wp_remote_post( STONEHENGE, array( 'timeout' => 25, 'sslverify' => false, 'body' => $parameters ) );
		    $license_data 	= json_decode(wp_remote_retrieve_body($response), true);
			return $license_data;
		}


		#===============================================
		public static function activate( $license_key, $plugin ) {
			$license_data 	= self::license_server( $license_key, 'activate_license', $plugin);
			$check_data 	= self::check($license_key, $plugin);
			return;
		}


		#===============================================
		public static function check( $license_key, $plugin ) {
			$license_data 	= self::license_server( $license_key, 'check_license', $plugin);
			$save_data 		= self::process_license_data( $license_data, $license_key, $plugin );
			return $license_data;
		}


		#===============================================
		public static function validate() {
			$plugin 		= self::$plugin;
			$license_key	= $plugin['license']['license_key'];
			$license_data 	= self::license_server( $license_key, 'check_license', $plugin);
			$save_data 		= self::process_license_data( $license_data, $license_key, $plugin );
			return;
		}


		#===============================================
		public static function deactivate( $license_key, $plugin ) {
			$license_data 	= self::license_server( $license_key, 'deactivate_license');
			delete_option( $plugin['license_slug'] );
			return;
		}


		#===============================================
		private static function process_license_data( $license_data, $license_key, $plugin ) {
			if( !$license_data['success'] ) {
				return;
			}

			$new_data 	= array();
			$data 		= array( 'success' => '', 'license' => '', 'expires' => '', 'customer_name' => '', 'customer_email' => '', 'license_limit' => '', 'site_count' => '', 'activations_left' => '', 'item_id' => '', 'error' => '');

			foreach($license_data as $key => $value ) {
				if( array_key_exists($key, $data) ) {
					$new_data[$key] = $value;
				}
			}
			$new_data['license_key'] = $license_key;

			if( !add_option( $plugin['license_slug'] , $new_data, '', 'no' ) ) {
				update_option( $plugin['license_slug'] , $new_data, 'no' );
			}
			return $new_data;
		}


		#===============================================
		public static function get_renewal_url( $plugin ) {
			$url = STONEHENGE .'checkout/?edd_license_key='. $plugin['license']['license_key'] . '&download_id=' . $plugin['license']['item_id'];
			return $url;
		}


		#===============================================
		public static function mask($input) {
			$masked =  str_repeat("*", strlen($input)-6) . substr($input, -6);
			return $masked;
		}
	}
}


#===============================================
if( !function_exists( 'start_stonehenge' ) ) {
	function start_stonehenge($plugin) {
		$start 	= Stonehenge_License::init($plugin);
		$init 	= $start->plugin;
		$check 	= Stonehenge_License::is_valid($init);
		return $check;
	}
}
