<?php
/************************************************************
* Plugin Name:	Events Manager – Ongoing Events
* Description:	Link single events together with one booking. Perfect for courses, classes, multi-day seminars, etc.
* Version:		1.0
* Author:  		Stonehenge Creations
* Author URI: 	https://www.stonehengecreations.nl/
* Plugin URI: 	https://www.stonehengecreations.nl/creations/stonehenge-em-ongoing-events/
* License URI: 	https://www.gnu.org/licenses/gpl-2.0.html
* Text Domain: 	stonehenge-em-ongoing-events
* Domain Path: 	/languages/
************************************************************/

// Exit if accessed directly
if( !defined('ABSPATH') ) exit;
if( !defined('STONEHENGE') ) define('STONEHENGE', 'https://www.stonehengecreations.nl/');

include_once(ABSPATH.'wp-admin/includes/plugin.php');


Class EM_Ongoing_Events {

	public function __construct() {
		add_action('em_ongoing_loaded', array($this, 'plugin_loaded'));
		if( start_stonehenge( $this->get_plugin_data() ) ) {
			do_action('em_ongoing_loaded');
		}
	}


	#===============================================
	# 	Define Plugin Data.
	#===============================================
	public static function get_plugin_data() {
		$wp 	= get_plugin_data( __FILE__ );
		$plugin = array(
			'name' 		=> 'Events Manager – Ongoing Events',
			'short' 	=> 'Ongoing Events',
			'icon' 		=> '&#x1F517;',
			'version' 	=> $wp['Version'],
			'text' 		=> $wp['TextDomain'],
			'slug' 		=> $wp['TextDomain'],
			'class' 	=> __CLASS__,
			'base' 		=> plugin_basename(__DIR__),
			'prio' 		=> 40,
		);
		$plugin['url'] 	= admin_url().'admin.php?page='.$plugin['slug'];
		return $plugin;
	}


	public static function plugin_loaded() {
		include('classes/class-metabox.php');
		include('classes/class-process.php');
		add_action('init', array($this, 'load_translations'));
		add_filter('em_cpt_event', array($this, 'add_hierarchy'), 10, 1);
		add_filter('em_ongoing_wildcards', array($this, 'replace_wildcards'), 10, 3);
	}


	#===============================================
	# 	Perform update.
	#===============================================
	public static function update_this_plugin() {
		// Do stuff.
		//$plugin = self::get_plugin_data();
	}


	#===============================================
	# 	Sanitize form inputs before saving to the database.
	#===============================================
	public static function sanitize_option_fields( $input ) {
		$sections 	= self::define_option_fields();
		$clean 		= Stonehenge_Plugin::sanitize_option_fields( $input, $sections );
		return $clean;
	}


	#===============================================
	# 	Make this plugin translatable.
	#===============================================
	public static function load_translations() {
		$plugin = self::get_plugin_data();
		$text 	= $plugin['text'];

		// Try included files first.
		$locale = apply_filters( 'plugin_locale', function_exists( 'determine_locale' ) ? determine_locale() : get_locale(), $text );
		$mofile = dirname( __FILE__ ) . '/languages/'. $text . '-' . $locale . '.mo';
		$loaded = load_textdomain( $text, $mofile );

		// Else, look for the WP Translations.
		if( !$loaded ) {
			$loaded = load_plugin_textdomain( $text, false, '/languages/' );
		}

		// MU Plugin perhaps?
		if( !$loaded ) {
			$loaded = load_muplugin_textdomain( $text, '/languages/' );
		}
	}


	#===============================================
	# 	Load Styles & Scripts.
	#===============================================
	public static function load_assets()  {
		$plugin = self::get_plugin_data();
		Stonehenge_Plugin::load_assets();

		wp_enqueue_style('em-ongoing-css', plugins_url('assets/stonehenge-em-ongoing-events.css', __FILE__), array(), $plugin['version'], 'all');
		wp_enqueue_script('em-ongoing-js', plugins_url('assets/stonehenge-em-ongoing-events.js', __FILE__), array('jquery'), $plugin['version'], false);
		wp_localize_script('em-ongoing-js', 'EMOE', self::localize_js_strings());
	}


	#===============================================
	# 	Create the Plugin options for this plugin.
	#===============================================
	public static function define_option_fields() {
		$plugin = self::get_plugin_data();
		$text 	= $plugin['text'];
		$sections['settings'] = array(
			'id' 		=> 'settings',
			'label'		=> __('Settings'),
			'fields' 	=> array(
				array(
					'id' 		=> 'intro',
					'label' 	=> __('What does this do?', $text),
					'type' 		=> 'intro',
					'default' 	=> sprintf('<div class="note"><strong>%s</strong><br>%s</div>', __('How do Ongoing Events work?', $text), __('This add-on lets you create single events that are linked together, but only require one booking for the user to attend all events. This can be useful if you are organizing classes, multi-day workshops or seminars, for example.', $text) .' ' .__('The idea is that you create one single event as the first day of this ongoing series. In that event you can specify the other dates and times for the underlying events.', $text) .'</p><br><p>'. __('When you publish the main event, the sub events will be created automatically. Their content will reflect that it is an ongoing sub event and link back to the booking form of the main event.', $text) .'</p><br><p>'. __('<b>Note:</b> Dates and times of the sub events can only be maintained in the main event.', $text) ),
				),
				array(
					'id' 		=> 'delete',
					'label' 	=> __('Delete Data', $text),
					'type' 		=> 'toggle',
					'required'	=> true,
					'helper' 	=> __('Automatically delete all data from your database when you uninstall this plugin?', $text),
					'default' 	=> 'no',
				),
			)
		);

		$sections['strings'] = array(
			'id' 		=> 'strings',
			'label'		=> __('Translatable Strings', $text),
			'fields' 	=> array(
				array(
					'id'		=> 'info',
					'label' 	=> '',
					'type' 		=> 'info',
					'default' 	=> __('Please specify below how you want certain fields of the sub events to be named. Your input will be used throughout this plugin in the Admin and Front-End.', $text),
				),
				array(
					'id' 		=> 'item',
					'label' 	=> sprintf( __('Label for %s', $text), __('Item', $text) ),
					'type' 		=> 'text',
					'default' 	=> __('Day'),
					'required' 	=> true,
					'helper' 	=> __('This indicates the Sub Event. Use something like "Day" or "Part".', $text),
				),
				array(
					'id' 		=> 'date',
					'label' 	=> sprintf( __('Label for %s', $text), __('Date') ),
					'type' 		=> 'text',
					'default' 	=> __('Date'),
					'required' 	=> true,
					'helper' 	=> self::show_settings_helper('Date Format'),
				),
				array(
					'id' 		=> 'time',
					'label' 	=> sprintf( __('Label for %s', $text), __('Time') ),
					'type' 		=> 'text',
					'default' 	=> __('Time'),
					'required' 	=> true,
					'helper' 	=> self::show_settings_helper('Time Format'),
				),

				array(
					'id' 		=> 'shortcode_title',
					'label' 	=> sprintf( __('Label for %s', $text), __('Table') ),
					'type' 		=> 'text',
					'default' 	=> 'Related Events',
					'required' 	=> true,
					'helper' 	=> __('This is the Header of the [ongoing-events] table.', $text),
				),
				array(
					'id' 		=> 'booking_notice',
					'label' 	=> __('Booking Notice', $text),
					'type' 		=> 'text',
					'default' 	=> 'Booking a single ticket will grant you access to all event dates listed above.',
					'required' 	=> true,
					'helper' 	=> __('This message is shown below the Ongoing Events Table to tell visitors they only have to book one time for all dates.', $text),
				),
				array(
					'id' 		=> 'wildcards',
					'label'		=> esc_html__('Wildcards', $text),
					'type' 		=> 'span',
					'default' 	=> sprintf('<strong>%s</strong>', esc_html__('You can use the following wildcards in sections below to target the specific information:', $text)) . '<br>'. self::show_wildcards(),
				),
				array(
					'id' 		=> 'sub_notice',
					'label' 	=> __('Sub Event Indication', $text),
					'type' 		=> 'text',
					'default' 	=> 'This is %SUB_LABEL% %SUB_COUNT% of %MAIN_NAME%.',
					'required' 	=> true,
				),
				array(
					'id' 		=> 'sub_content',
					'label' 	=> __('Sub Event Content', $text),
					'type' 		=> 'editor',
					'default' 	=> '<h2>This is %SUB_LABEL% %SUB_COUNT% of %MAIN_NAME%, which starts on %MAIN_DATE%.</h2>
<p>[ongoing_events]</p>
<h3>Bookings</h3>
[event post_id="%MAIN_ID%"]#_BOOKINGFORM[/event]',
					'required' 	=> true,
				),
				array(
					'id' 		=> 'sub_excerpt',
					'label' 	=> __('Sub Event Excerpt', $text),
					'type' 		=> 'textarea',
					'default' 	=> 'This is %SUB_LABEL% %SUB_COUNT% of %MAIN_NAME%, starting on %MAIN_DATE%.',
					'required' 	=> true,
				),
			)
		);
		return $sections;
	}


	#===============================================
	# 	Show available wildcards in the Options Page.
	#===============================================
	public static function show_wildcards() {
		$plugin = self::get_plugin_data();
		$text 	= $plugin['text'];
		$show 	= '<div id="wildcards" class="note">';
		$show 	.= sprintf('%s<br>%s<br>%s<br>%s<br>%s<br><br>%s<br>%s<br>%s<br>%s<br>%s</div>',
'%MAIN_ID% &rArr; '. esc_html__('Displays post ID of the Main Event.', $text),
'%MAIN_NAME% &rArr; '. esc_html__('Displays event name of the Main Event.', $text),
'%MAIN_DATE% &rArr; '. esc_html__('Displays start date of the Main Event.', $text),
'%MAIN_START% &rArr; '. esc_html__('Displays start time of the Main Event.', $text),
'%MAIN_END% &rArr; '. esc_html__('Displays end time of the Main Event.', $text),
'%SUB_LABEL% &rArr; '. esc_html__('Displays label of each sub event (first field above).', $text),
'%SUB_COUNT% &rArr; '. esc_html__('Displays the incremented number of this specific Sub Event.', $text),
'%SUB_DATE% &rArr; '. esc_html__('Displays the start date of this specific Sub Event.', $text),
'%SUB_START% &rArr; '. esc_html__('Displays the start time of this specific Sub Event.', $text),
'%SUB_END% &rArr; '. esc_html__('Displays the end time of this specific Sub Event.', $text)
);
		return $show;

	}

	#===============================================
	# 	Handle wildcards.
	#===============================================
	public static function replace_wildcards($string, $event, $parent_id) {
		$plugin 			= self::get_plugin_data();
		$get 				= self::translate();
		$EM_Main 			= new EM_Event($parent_id, 'post_id');
		$wildcards 			= array(
			'%MAIN_ID%' 		=> $parent_id,
			'%MAIN_NAME%'		=> $EM_Main->event_name,
			'%MAIN_DATE%' 		=> date_i18n($get['date_format'], strtotime($EM_Main->start_date)),
			'%MAIN_START'		=> date_i18n($get['time_format'], strtotime($EM_Main->start_time)),
			'%MAIN_END%' 		=> date_i18n($get['time_format'], strtotime($EM_Main->end_time)),
			'%SUB_LABEL%'		=> $get['item'],
			'%SUB_COUNT%'		=> $event['day'],
			'%SUB_DATE%' 		=> date_i18n($get['date_format'], strtotime($event['date'])),
			'%SUB_START%' 		=> date_i18n($get['time_format'], strtotime($event['start'])),
			'%SUB_END%'			=> date_i18n($get['time_format'], strtotime($event['end']))
		);

		foreach( $wildcards as $wildcard => $value ) {
			$string = str_replace( $wildcard, $value, $string );
		}
		return $string;
	}


	#===============================================
	# 	Create an easy link to EM Settings Page.
	#===============================================
	public static function show_settings_helper($type) {
		$plugin 	= self::get_plugin_data();
		$text 		= $plugin['text'];
		$url 		= esc_url_raw( admin_url() . 'edit.php?post_type=event&page=events-manager-options#formats' );
		$link 		= sprintf('<a href=%1$s target="_blank">%2$s</a>', $url, __('Events Manager Settings', $text) );
		$message 	= wp_kses_allowed( sprintf( _x('This plugin uses the %1$s in your %2$s.', 'This plugin uses the formatting in your settings', $text),
			esc_html__($type), $link ) );
		return $message;
	}


	#===============================================
	# 	Use Settings to easily translate output.
	#===============================================
	public static function translate() {
		$plugin 				= self::get_plugin_data();
		$text 					= $plugin['text'];
		$saved 					= get_option( $plugin['slug'] ); 		// Fetch strings defined by user in options.php.
		$get 					= $saved['strings'];
		$add['remove']			= __('Remove');
		$add['add'] 			= __('Add');
		$add['date_format_js']	= get_option('dbem_date_format_js');
		$add['date_format']		= get_option('dbem_date_format');
		$add['time_format'] 	= get_option('dbem_time_format');
		$add['year_range'] 		= '"'. date('Y') .':'. (date('Y') + 5) .'"';
		$add['maintain']		= __('These details are maintained in the Main Event.', $text );
		$add['admin_main']		= sprintf( '<strong>%s</strong><br>%s %s',
		 	esc_html__('This is the main event of an ongoing event series.', $text ),
		 	esc_html__('Dates and booking are maintained here.', $text ),
		 	esc_html__('If you delete this event, all linked sub-events will be deleted as well.', $text)
		);
		$translate = array_merge($get, $add);
		return $translate;
	}


	#===============================================
	# 	Make JavaScript strings translatable.
	#===============================================
	public static function localize_js_strings() {
		$plugin 	= self::get_plugin_data();
		$translate 	= self::translate();
		$localize 	= array(
			'get' => $translate,
		);
		return $localize;
	}


	#===============================================
	# 	Display custom Admin Notices.
	#===============================================
	public static function show_notice($message, $class, $dismissible) {
		$plugin = self::get_plugin_data();
		if( $dismissible ) {
			$dismissible = 'is-dismissible';
		}
		$notice = wp_kses_allowed( sprintf( '<div class="notice notice-%s %s"><p>%s</p></div>', $class, $dismissible, __($message, $plugin['text']) ) );
		return $notice;
	}

	#===============================================
	# 	Add Post Hierarchy to EM Events.
	#===============================================
	public static function add_hierarchy($args) {
		if( !$args['hierarchical'] ) {
			$args['hierarchical'] = true;
			flush_rewrite_rules();
			return $args;
		}
		return $args;
	}


	#===============================================
	# 	Shortcode: Output HTML table to list al ongoing events.
	#===============================================
	public static function show_ongoing_events_table() {
		$plugin 		= parent::get_plugin_data();
		$get 			= parent::translate();

		if( !event_is_ongoing() ) {
			return __('This is not an ongoing event.', $plugin['text']);
		}

		$parent_id 		= event_get_parent_id();
		$linked_events 	= event_get_linked_events();

		// First construct line 1 for Day 1 (Main Event).
		$EM_Main 		= em_get_event($parent_id, 'post_id');
		$main_url 		= get_permalink($EM_Main->post_id);
		$row_one 		= $EM_Main->output("<tr><td><strong><a href='{$main_url}' title='#_EVENTNAME'>{$get['item']} 1</a></strong></td><td>#_EVENTDATES</td><td>#_EVENTTIMES</td></tr>");

		// Start output.
		$table 	= null;
		$table 	.= '<h3>' . $get['shortcode_title'] . '</h3>';
		$table	.= '<table class="ongoing-events-table">
			<thead>
				<th></th>
				<th>'. $get['date'] .'</th>
				<th>'. $get['time'] .'</th>
			</thead>
			<tbody>';
		$table .= $row_one;

		// Loop through sub-events.
		foreach( $linked_events as $linked_event ) {

			$event 	= new EM_Event($linked_event->_event_id);
			$nr 	= $event->event_attributes['ongoing_day'];
			$url 	= get_permalink($event->post_id);
			$row 	= "<tr><td><strong><a href='{$url}' title='#_EVENTNAME'>{$get['item']} {$nr}</a></strong></td><td>#_EVENTDATES</td><td>#_EVENTTIMES</td></tr>";

			$table 	.= $event->output($row);
		}
		$table 	.= '</tbody></table>';
		$table 	.= '<p class="ongoing-events-notice">' . $get['booking_notice'] . '</p>';
		return $table;
	}

} // End class.


#===============================================
add_action('plugins_loaded', function() {
	if( !is_plugin_active('events-manager/events-manager.php') ) {
		deactivate_plugins(plugin_basename(__FILE__));
		wp_die('<b>EM Ongoing Events</b> needs Events Manager to be installed and activated.<br>This add-on has been deactivated...', 'Plugin dependency check', array('back_link' => true));
	}
	else {
		require_once('config/init.php');
		new EM_Ongoing_Events();
	}
}, 10);

#===============================================
add_action('activated_plugin', function($plugin) {
	if( $plugin == plugin_basename( __FILE__ ) ) {
		exit( wp_redirect( admin_url().'admin.php?page=stonehenge-em-ongoing-events' ) );
	}
});


#===============================================
require('config/server/update-checker.php');
$base = plugin_basename( __DIR__ );
$UpdateChecker = Puc_v4_Factory::buildUpdateChecker(
	STONEHENGE . 'update-checker/?action=get_metadata&slug=' . $base,
	__FILE__,
	$base
);
